using System.Text;

#pragma warning disable IDE0130 // Namespace does not match folder structure
namespace ArmaRAMDb
#pragma warning restore IDE0130 // Namespace does not match folder structure
{
    internal class Utils
    {
        public static string Base64Decode(string encodedString)
        {
            byte[] data = Convert.FromBase64String(encodedString);
            return Encoding.UTF8.GetString(data);
        }

        public static string Base64Encode(string plainText)
        {
            byte[] data = Encoding.UTF8.GetBytes(plainText);
            return Convert.ToBase64String(data);
        }

        public static string BuildArray(string text)
        {
            string str = $"[\"{text}\"]";
            return str;
        }

        public static List<string> SplitIntoChunks(string data, int maxChunkSize)
        {
            List<string> chunks = [];

            if (data.StartsWith("[") && data.EndsWith("]"))
            {
                string innerData = data[1..^1];
                
                List<string> elements = [];
                int depth = 0;
                int startPos = 0;
                
                for (int i = 0; i < innerData.Length; i++)
                {
                    char c = innerData[i];
                    
                    if (c == '[') depth++;
                    else if (c == ']') depth--;
                    
                    else if (c == ',' && depth == 0)
                    {
                        elements.Add(innerData[startPos..i]);
                        startPos = i + 1;
                    }
                }
                
                if (startPos < innerData.Length)
                {
                    elements.Add(innerData[startPos..]);
                }
                
                StringBuilder currentChunk = new StringBuilder();
                
                foreach (string element in elements)
                {
                    if (currentChunk.Length > 0 && 
                        Encoding.UTF8.GetByteCount(currentChunk.ToString() + "," + element) > maxChunkSize)
                    {
                        chunks.Add(currentChunk.ToString());
                        currentChunk.Clear();
                    }
                    
                    if (currentChunk.Length > 0)
                    {
                        currentChunk.Append(",");
                    }
                    
                    currentChunk.Append(element);
                }
                
                if (currentChunk.Length > 0)
                {
                    chunks.Add(currentChunk.ToString());
                }
            }
            else
            {
                int bytesProcessed = 0;
                while (bytesProcessed < Encoding.UTF8.GetByteCount(data))
                {
                    int charCount = 0;
                    while (bytesProcessed + Encoding.UTF8.GetByteCount(data.Substring(bytesProcessed, Math.Min(charCount + 1, data.Length - bytesProcessed))) <= maxChunkSize && 
                           bytesProcessed + charCount < data.Length)
                    {
                        charCount++;
                    }
                    
                    chunks.Add(data.Substring(bytesProcessed, charCount));
                    bytesProcessed += charCount;
                }
            }
            
            return chunks;
        }

        public static void CheckByteCount(string uniqueId, string data, string function, string entity, bool call, int bufferSize)
        {
            if (Encoding.UTF8.GetByteCount(data) <= bufferSize)
            {
                if (!data.StartsWith('['))
                    data = BuildArray(data);
                Main.Log($"Single chunk data: {data}", "debug");

                string dataAsString = $"[\"{uniqueId}\",\"{function}\",{call.ToString().ToLower()},{data},\"{entity}\"]";
                Main.Callback("ArmaRAMDb", "ramdb_db_fnc_handler", dataAsString);
            }
            else
            {
                if (!data.StartsWith('[') || !data.EndsWith(']'))
                {
                    data = BuildArray(data);
                }
                
                List<string> elements = ParseArrayElements(data);
                
                int totalChunks = CalculateChunks(elements, bufferSize);
                
                for (int chunkIndex = 0; chunkIndex < totalChunks; chunkIndex++)
                {
                    List<string> chunkElements = GetChunkElements(elements, chunkIndex, totalChunks);
                    
                    string chunkData = "[" + string.Join(",", chunkElements) + "]";
                    
                    string escapedChunkData = chunkData.Replace("\"", "\"\"");
                    
                    string chunkAsString = $"[\"{uniqueId}\",\"{function}\",{chunkIndex+1},{totalChunks},\"{escapedChunkData}\",{call.ToString().ToLower()},\"{entity}\"]";
                    
                    Main.Log($"Sending chunk {chunkIndex+1}/{totalChunks}: {chunkAsString}", "debug");
                    Main.Callback("ArmaRAMDb", "ramdb_db_fnc_fetch", chunkAsString);
                }
            }
        }

        private static List<string> ParseArrayElements(string arrayString)
        {
            List<string> elements = [];
            
            string content = arrayString[1..^1].Trim();
            
            if (string.IsNullOrEmpty(content))
                return elements;
            
            int startPos = 0;
            int nestLevel = 0;
            bool inQuotes = false;
            
            for (int i = 0; i < content.Length; i++)
            {
                char c = content[i];
                
                if (c == '\\' && i + 1 < content.Length && content[i + 1] == '"')
                {
                    i++;
                    continue;
                }
                
                if (c == '"')
                    inQuotes = !inQuotes;
                
                if (!inQuotes)
                {
                    if (c == '[') nestLevel++;
                    else if (c == ']') nestLevel--;
                    
                    if (nestLevel == 0 && c == ',')
                    {
                        elements.Add(content[startPos..i].Trim());
                        startPos = i + 1;
                    }
                }
            }
            
            if (startPos < content.Length)
            {
                elements.Add(content[startPos..].Trim());
            }
            
            return elements;
        }

        private static int CalculateChunks(List<string> elements, int bufferSize)
        {
            int chunks = 1;
            int currentSize = 2;
            
            foreach (string element in elements)
            {
                int elementSize = Encoding.UTF8.GetByteCount(element) + 1;
                
                if (currentSize + elementSize > bufferSize)
                {
                    chunks++;
                    currentSize = 2 + elementSize;
                }
                else
                {
                    currentSize += elementSize;
                }
            }
            
            return chunks;
        }

        private static List<string> GetChunkElements(List<string> allElements, int chunkIndex, int totalChunks)
        {
            int elementsPerChunk = (int)Math.Ceiling((double)allElements.Count / totalChunks);
            int startIdx = chunkIndex * elementsPerChunk;
            int count = Math.Min(elementsPerChunk, allElements.Count - startIdx);
            
            List<string> result = [];
            for (int i = 0; i < count; i++)
            {
                if (startIdx + i < allElements.Count)
                    result.Add(allElements[startIdx + i]);
            }
            
            return result;
        }
    }
}